'use strict';

var HANDLEBARS_PREFIX = 'handlebarsTemplate_';

var noop = function noop(node) {
  return '';
};

function addParentContext(params) {
  return params.concat(['\'__parent\' => $params']);
}

function ifStatement(node) {
  return '(' + node.condition + ' ? (' + (node.ifBody.length ? node.ifBody.join(' . ') : '\'\'') + ') : (' + (node.elseBody.length ? node.elseBody.join(' . ') : '\'\'') + '))';
}

function unlessStatement(node) {
  return ifStatement({
    ifBody: node.unlessBody,
    type: 'handlebars',
    subType: 'IF',
    elseBody: node.elseBody,
    condition: '!' + node.condition
  });
}

function eachStatement(node) {
  var eachExpr = 'implode(\'\', array_map(function($params) {\n    return ' + (node.eachBody.length > 0 ? node.eachBody.join(' . ') : '\'\'') + ';\n  }, ' + node.iterator + '))';

  if (node.elseBody.length > 0) {
    return ifStatement({
      ifBody: [eachExpr],
      type: 'handlebars',
      subType: 'IF',
      elseBody: node.elseBody,
      condition: node.iterator
    });
  }
  return eachExpr;
}

function evaluationStatement(node) {
  if ((+node.value).toString() === node.value) {
    return node.value;
  }

  if (node.value === 'true' || node.value === 'false') {
    return node.value;
  }

  var looks = node.value.split('.');

  return '$params' + looks.map(function (el) {
    return '[\'' + el + '\']';
  }).join('');
}

function lookupStatement(node) {
  return '$params' + node.value.map(function (call) {
    if (call === 'this' || call === '.') {
      return '';
    }
    return '[\'' + call + '\']';
  }).join('');
}

var PhpAdapter = {
  htmlBlockNode: function htmlBlockNode(node) {
    var attributes = node.attributes.length > 0 ? ' \' . ' + node.attributes.join(' . ') + ' . \'' : '';
    if (node.children.length > 0) {
      return '\'<' + node.openTag + attributes + '>\'\n  . ' + node.children.join(' .\n') + ' .\n\'</' + node.closeTag + '>\'';
    } else {
      return '\'<' + node.openTag + attributes + '/>\'';
    }
  },
  htmlSingleNode: function htmlSingleNode(node) {
    if (node.attributes.length > 0) {
      return '\'<' + node.tag + ' \' . ' + node.attributes.join(' . ') + ' . \'/>\'';
    } else {
      return '\'<' + node.tag + '/>\'';
    }
  },
  htmlComment: function htmlComment(node) {
    return '\'<!--' + node.value + '-->\'';
  },
  text: function text(node) {
    return '\'' + node.value + '\'';
  },
  handlebars: {
    IF: function IF(node) {
      return ifStatement(node);
    },
    EACH: function EACH(node) {
      return eachStatement(node);
    },
    UNLESS: function UNLESS(node) {
      return unlessStatement(node);
    },

    SAFEEVALUATION: function SAFEEVALUATION(node) {
      return 'hbarsEscape(' + evaluationStatement(node) + ')';
    },
    SINGLEEVALUATION: function SINGLEEVALUATION(node) {
      return evaluationStatement(node);
    },

    LOOKUPSINGLE: function LOOKUPSINGLE(node) {
      return lookupStatement(node);
    },

    UNSAFEEVALUATION: function UNSAFEEVALUATION(node) {
      return evaluationStatement(node);
    },

    NAMEDPARAMETER: function NAMEDPARAMETER(node) {
      var value = node.value.map(function (val) {
        return PhpAdapter.handlebars[val.subType](val);
      }).join(' ');
      return '\'' + node.key + '\' => ' + value;
    },

    PARTIAL: function PARTIAL(node) {
      var params = addParentContext(node.parameters);
      if (node.children.length > 0) {
        params.push('\'@page-block\' => ' + node.children.join(' . '));
      }

      return 'handlebarsTemplate_' + node.value + '(array(' + params.join(', \n') + '))';
    },

    SINGLEPARTIALS: function SINGLEPARTIALS(node) {
      if (node.value === '@page-block') {
        return '$params[\'@page-block\']';
      }
      var params = addParentContext(node.parameters);
      return 'handlebarsTemplate_' + node.value + '(array(' + params.join(', \n') + '))';
    },
    SIMPLEVALUE: function SIMPLEVALUE(value) {
      return value.value;
    },
    ATTRSINGLECOMMENTS: function ATTRSINGLECOMMENTS(node) {
      return '\'\'';
    },
    SINGLECOMMENTS: function SINGLECOMMENTS(node) {
      return '\'\'';
    },
    ATTRLOOKUPSINGLE: function ATTRLOOKUPSINGLE(node) {
      return lookupStatement(node);
    },
    ATTRSINGLEEVALUATION: function ATTRSINGLEEVALUATION(node, shouldWrap) {
      if (shouldWrap) {
        return 'hbarsEscape(' + evaluationStatement(node) + ')';
      } else {
        return evaluationStatement(node);
      }
    },
    ATTRUNSAFESINGLEEVALUATION: function ATTRUNSAFESINGLEEVALUATION(node) {
      return evaluationStatement(node);
    }

    // GENERICBLOCK: noop,
    // GENERICSINGLE: noop,
    // UNSAFEGENERICSINGLE: noop,
    // SAFEREFERENCEEVALUATION: noop,
    // UNSAFEREFERENCEEVALUATION: noop,
    // ATTRGENERICBLOCK: noop,
    // ATTRGENERICSINGLE: noop,
    // REFERENCEEVALUATION: NodeProcessor,
    // ATTRUNSAFEGENERICSINGLE: noop,
    // ATTRREFERENCEEVALUATION: noop,
    // ATTRUNSAFEREFERENCEEVALUATION: noop,
  },
  keyValue: function keyValue(attr) {
    var value = attr.value.filter(function (value) {
      return !value.subType || value.subType !== 'extraSpaces';
    }).map(function (value) {
      return value.value;
    }).join(' ');
    return '\'' + attr.key + '="' + value + '"\'';
  },
  singleKey: function singleKey(attr) {
    var value = attr.value instanceof Array ? attr.value.join('') : attr.value;
    return '\'' + value + '\'';
  },
  singleValue: function singleValue(node) {
    return '';
  },
  topLevel: function topLevel(node) {
    return '\nfunction ' + (HANDLEBARS_PREFIX + node.name) + '($params) {\n  return ' + node.statements.join(' . ') + ';\n}';
  }
};

module.exports = PhpAdapter;